#include <stdlib.h>
#include <libebook/e-book.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>

#include "usbook.h"

static gchar header1[] = {0x55, 0xAA, 0x33, 0xCC, 0xFF, 0xEE, 0xDD, 0xCC, 0x01, 0x02, \
	                       0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x00, 0x32, 0x00, 0x00};
static gchar header2[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xA1, 0x26, 0x47, 0xF1};

/*
 * UTILITY Functions
 */

static int towrite = 0;

size_t usbook_utils_write(int fd, void *data, size_t size)
{
	size_t w, written = 0;
	
	do {
		do {
			w = write (fd, data + written, size - written);
		} while (w == -1 && errno == EINTR);
		
		if (w > 0)
			written += w;
	} while (w != -1 && written < size);
	
	return written;
}

/*
 * USBookContact management functions
 */

USBookContact *usbook_contact_new()
{
	USBookContact *usbookContact = g_malloc0( sizeof(USBookContact) );
	
	return usbookContact;
}

void usbook_contact_free(USBookContact *usbookContact)
{
	g_free ( (gpointer) usbookContact );
}

size_t usbook_contact_save(int fd, USBookContact *usbookContact)
{
	return usbook_utils_write(fd, (void *) usbookContact, sizeof(USBookContact) );
}

/*
 * USBook management functions
 */

USBook *usbook_book_new()
{
	USBook *usbook = g_malloc( sizeof(USBook) );
	
	usbook->header1 = header1;
	usbook->contacts_count = 0;
	usbook->header2 = header2;
	usbook->contacts = NULL;
		
	return usbook;
}

void usbook_book_free(USBook *usbook)
{
	/* FREE all contacts */
	g_list_foreach (usbook->contacts, (GFunc)g_free, NULL);
	g_list_free (usbook->contacts);
	g_free( (gpointer) usbook );
}

/*
 * Conversions functions
 */

USBookContact *usbook_contact_convert (EContact *contact)
{
	USBookContact *usbookContact = usbook_contact_new();
	gchar *fieldData;
	
	fieldData = e_contact_get_const (contact, E_CONTACT_NAME_OR_ORG);
	if ( fieldData )
		g_memmove(usbookContact->name,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->name) - 1) );

	fieldData = e_contact_get_const (contact, E_CONTACT_PHONE_PRIMARY);
	if ( ! fieldData )
		fieldData = e_contact_get_const (contact, E_CONTACT_PHONE_HOME);
	if ( ! fieldData )
		fieldData = e_contact_get_const (contact, E_CONTACT_PHONE_BUSINESS);
	if ( ! fieldData )
		fieldData = e_contact_get_const (contact, E_CONTACT_PHONE_COMPANY);	
	if ( fieldData ) 
		g_memmove(usbookContact->telephone_number,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->telephone_number) - 1) );

	fieldData = e_contact_get_const (contact, E_CONTACT_PHONE_MOBILE);
	if ( fieldData )
		g_memmove(usbookContact->mobile_number,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->mobile_number) ) );

	fieldData = e_contact_get_const (contact, E_CONTACT_EMAIL_1);
	if ( fieldData )
		g_memmove(usbookContact->email,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->email) ) );

	fieldData = e_contact_get_const (contact, E_CONTACT_IM_JABBER_WORK_1);
	if ( fieldData )
		g_memmove(usbookContact->instant_msg_id,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->instant_msg_id) ) );

	fieldData = e_contact_get_const (contact, E_CONTACT_PHONE_BUSINESS_FAX);
	if ( ! fieldData )
		fieldData = e_contact_get_const (contact, E_CONTACT_PHONE_HOME_FAX);
	if ( ! fieldData )
		fieldData = e_contact_get_const (contact, E_CONTACT_PHONE_OTHER_FAX);
	if ( fieldData )
		g_memmove(usbookContact->fax_number,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->fax_number) ) );
	
	return usbookContact;
}

USBook *usbook_book_convert (EBook *book)
{
	EBookQuery *query;
	gboolean status;
	GList *cards, *c;
	USBook *usbook;
	USBookContact *usbookContact;
	
	/* Init USBook */
	usbook = usbook_book_new();
	query = e_book_query_field_exists (E_CONTACT_FULL_NAME);

	status = e_book_get_contacts (book, query, &cards, NULL);

	e_book_query_unref (query);

	if (status == FALSE) {
		printf ("error %d getting card list\n", status);
		exit(0);
	}
	
	for (c = cards; c; c = c->next) {
		EContact *contact = E_CONTACT (c->data);

		usbookContact = usbook_contact_convert (contact);

		usbook->contacts = g_list_prepend( usbook->contacts, usbookContact );
		g_object_unref (contact);
	}
	
	/* Debug
	char *fieldData;
	
	usbookContact = usbook_contact_new();
	fieldData="nome567890123456";
	g_memmove(usbookContact->name,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->name) ) );
	fieldData="telefono901234567890123456789012";
	g_memmove(usbookContact->telephone_number,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->telephone_number) ) );
	fieldData="cellulare01234567890123456789012";
	g_memmove(usbookContact->mobile_number,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->mobile_number) ) );
	fieldData="email678901234567890123456789012";
	g_memmove(usbookContact->email,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->email) ) );
	fieldData="qq345678901234567890123456789012";
	g_memmove(usbookContact->instant_msg_id,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->instant_msg_id) ) );
	fieldData="fax45678901234567890123456789012";
	g_memmove(usbookContact->fax_number,  fieldData, MIN( strlen(fieldData), sizeof(usbookContact->fax_number) ) );
	usbook->contacts = g_list_prepend( usbook->contacts, usbookContact );	
	 END Debug */
	
	usbook->contacts_count = g_list_length(usbook->contacts);
	g_list_free (cards);
	return usbook;
}

/*
 * USBook FILE I/O Functions
 */

gboolean usbook_book_save(const gchar *filename, USBook *usbook)
{
	int fd;
	GList *contact;
	
	/* Opening file */
	towrite = 0x31F4;
	fd = g_open (filename, O_WRONLY | O_CREAT | O_TRUNC, 0600);
	if (fd == -1) 
		return FALSE;
	
	/* Headers */
	towrite -= usbook_utils_write(fd, (void *) usbook->header1, 0x14);
	towrite -= usbook_utils_write(fd, (void *) &usbook->contacts_count, 0x01 );
	towrite -= usbook_utils_write(fd, (void *) usbook->header2, 0x0B );
	
	/* Contacts */
	contact =  g_list_first( usbook->contacts );
	while ( contact != NULL )
	{
		towrite -= usbook_contact_save(fd, (USBookContact *) contact->data);
		contact = g_list_next(contact);
	}
	
	/* Fill file up to 0x31F3 */
	void *filler = g_malloc0(towrite);
	usbook_utils_write(fd, filler, towrite);
	g_free(filler);
	/* Closing file */
	
	close(fd);
	return TRUE;
}
